ROLES = {
    "top": "top",
    "jungle": "jungle",
    "mid": "mid",
    "bottom": "adc",
    "utility": "support",
}

SPELLS = {
    "SummonerBarrier": "21",
    "SummonerBoost": "1",
    "SummonerCherryFlash": "2202",
    "SummonerCherryHold": "2201",
    "SummonerDot": "14",
    "SummonerExhaust": "3",
    "SummonerFlash": "4",
    "SummonerHaste": "6",
    "SummonerHeal": "7",
    "SummonerMana": "13",
    "SummonerPoroRecall ": "30",
    "SummonerPoroThrow": "31",
    "SummonerSmite": "11",
    "SummonerSnowURFSnowball_Mark": "39",
    "SummonerSnowball": "32",
    "SummonerTeleport": "12",
    "Summoner_UltBookPlaceholder": "54",
    "Summoner_UltBookSmitePlaceholder": "55",
}



"""LeagueWizard main entry point."""

import asyncio
import os
import tempfile
import threading
import urllib
from pathlib import Path
from typing import Any

import pystray
from loguru import logger
from PIL import Image

from leaguewizard.core import start
from leaguewizard.exceptions import LeWizardGenericError

Path("logs").mkdir(parents=True, exist_ok=True)
logger.add("logs/leaguewiz_log.log", rotation="1 MB")


def to_tray() -> Any:
    """Create the tray icon with exit action.

    Returns:
        Any: Must be a pystray Icon object.
    """
    dest = f"{tempfile.gettempdir()}\\logo.png"
    urllib.request.urlretrieve(
        "https://github.com/amburgao/leaguewizard/blob/main/.github/images/logo.png?raw=true",
        dest,
    )
    return pystray.Icon(
        (0, 0),
        icon=Image.open(dest),
        menu=pystray.Menu(pystray.MenuItem("Exit", lambda icon, item: os._exit(0))),
    )


def main() -> None:
    """LeagueWizard main entry point function."""
    import socket

    s = socket.socket()
    try:
        s.bind(("127.0.0.1", 54321))
    except OSError as e:
        raise LeWizardGenericError(
            message="Another instance is already running",
            show=True,
            title="Error!",
            exit=True,
        ) from e

    tray = to_tray()
    tray_thread = threading.Thread(target=tray.run, daemon=True)
    tray_thread.start()

    asyncio.run(start())
    tray.stop()


if __name__ == "__main__":
    main()



"""Exceptions module for LeWizard."""

import sys


class LeWizardGenericError(Exception):
    """Base custom exception error for LeagueWizard."""

    def __init__(
        self, message: str, show: bool = False, title: str = "", exit: bool = False
    ) -> None:
        super().__init__(message)
        if show:
            from tkinter import messagebox

            messagebox.showerror(title=title, message=message)
        if exit:
            sys.exit()



"""Mobalytics handler module."""

import re
from typing import Any

import aiohttp
from async_lru import alru_cache
from loguru import logger
from selectolax.parser import HTMLParser

from leaguewizard.constants import SPELLS
from leaguewizard.exceptions import LeWizardGenericError
from leaguewizard.models import (
    Block,
    Item,
    ItemSet,
    Payload_ItemSets,
    Payload_Perks,
    Payload_Spells,
)


def _build_url(champion_name: str, role: str | None) -> str:
    base_url = "https://mobalytics.gg/lol/champions"
    return (
        f"{base_url}/{champion_name}/build/{role}"
        if role != "" and role is not None
        else f"{base_url}/{champion_name}/aram-builds"
    )


async def _get_html(url: str, client: aiohttp.ClientSession) -> HTMLParser:
    try:
        response = await client.get(url)
        if response.status >= 400:
            raise ConnectionError
        raw_html = await response.text()
        return HTMLParser(raw_html)
    except (aiohttp.ClientResponseError, ConnectionError) as e:
        logger.error(e)
        raise LeWizardGenericError("_get_html returned None.") from e


@alru_cache
async def get_mobalytics_info(
    champion_name: str,
    role: str,
    conn: aiohttp.ClientSession,
    champion_id: int,
    summoner_id: int,
) -> Any:
    """TODO."""
    try:
        page_url = _build_url(champion_name, role)
        tree = await _get_html(page_url, conn)
        if tree is None:
            pass
        skill_order = tree.css(".m-m4se9")
        skills = []
        for node in skill_order:
            skill_attr = node.text()
            skills.append(skill_attr)
        skills_string = " > ".join(skills)
        nodes = tree.css(Payload_ItemSets.itemsets_css)
        blocks: list[Block] = []
        for node in nodes:
            block_name_node = node.css_first("h4")
            if len(blocks) == 0:
                block_name = skills_string
            else:
                block_name = block_name_node.text() if block_name_node else ""
            items_node = node.css(".m-5o4ika")
            block_items: list[Item] = []
            for item_node in items_node:
                item = item_node.attributes.get("src")
                matches = re.search("(\\d+)\\.png", item) if item else None
                if matches is not None:
                    block_items.append(Item(1, matches.group(1)))
            block = Block(block_items, block_name)
            blocks.append(block)
        nodes = tree.css(".m-1eeoc06")
        situational_block_items: list[Item] = []
        for node in nodes:
            item = node.attributes.get("src")
            matches = re.search("(\\d+)\\.png", item) if item else None
            if matches is not None:
                situational_block_items.append(Item(1, matches.group(1)))
        block = Block(situational_block_items, "Situational Items")
        blocks.append(block)
        itemsets = ItemSet(
            [champion_id],
            blocks,
            f"{champion_name} ({role})"
            if role is not None and role != ""
            else f"{champion_name} (ARAM)",
        )
        itemsets_payload = Payload_ItemSets(
            accountId=summoner_id,
            itemSets=[itemsets],
            timestamp=0,
        )

        nodes = tree.css(Payload_Perks.main_perks_css)
        main_perks = []
        selected_perks = []
        for node in nodes:
            src = node.attributes.get("src")
            matches = re.search("/(\\d+)\\.svg", src) if src else None
            if matches:
                main_perks.append(int(matches.group(1)))
        for css in Payload_Perks.selected_perks_css:
            nodes = tree.css(css)
            for node in nodes:
                src = node.attributes.get("src")
                matches = re.search("/(\\d+)(\\.svg|\\.png)\\b", src) if src else None
                if matches:
                    selected_perks.append(int(matches.group(1)))
        perks_payload = Payload_Perks(
            name=f"{champion_name} - {role}"
            if role is not None and role != ""
            else f"{champion_name} - ARAM",
            current=True,
            primaryStyleId=int(main_perks[0]),
            subStyleId=int(main_perks[1]),
            selectedPerkIds=selected_perks,
        )

        nodes = tree.css(Payload_Spells.spells_css)
        spells_ids = []
        for node in nodes:
            src = node.attributes.get("src")
            matches = re.search("(\\w+)\\.png", src) if src else None
            if matches:
                spells_ids.append(SPELLS[matches[1]])
        spells_payload = Payload_Spells(
            selectedSkinId=champion_id,
            spell1Id=int(spells_ids[0]),
            spell2Id=int(spells_ids[1]),
        )
        logger.debug(f"Added to cache: {champion_name}")
        return itemsets_payload, perks_payload, spells_payload
    except (TypeError, AttributeError, ValueError, LeWizardGenericError) as e:
        logger.exception(e)
        pass



from __future__ import annotations

import asyncio
import base64
import contextlib
import json
import ssl
from pathlib import Path
from typing import Any

import aiohttp
import psutil
import websockets
from async_lru import alru_cache
from loguru import logger

from leaguewizard.constants import ROLES
from leaguewizard.mobalytics import get_mobalytics_info
from leaguewizard.models import (
    Payload_ItemSets,
    Payload_Perks,
    Payload_Spells,
)

_last_champion_id = None


RIOT_CERT = Path("./riotgames.pem").resolve()

context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
context.load_verify_locations(RIOT_CERT)
context.check_hostname = False


def _lcu_lockfile(league_exe: str) -> Path:
    if not Path(league_exe).exists():
        msg = "LeagueClient.exe not running or not found."
        raise ProcessLookupError(msg)
    league_dir = Path(league_exe).parent
    return Path(league_dir / "lockfile")


def _lcu_wss(lockfile: Path) -> dict[str, str]:
    with lockfile.open() as f:
        content = f.read()
    parts = content.split(":")

    port = parts[2]
    wss = f"wss://127.0.0.1:{port}"
    https = f"https://127.0.0.1:{port}"

    auth_key = parts[3]
    raw_auth = f"riot:{auth_key}"
    auth = base64.b64encode(bytes(raw_auth, "utf-8")).decode()
    return {"auth": auth, "wss": wss, "https": https}


@alru_cache
async def _get_champion_list(client: aiohttp.ClientSession) -> Any:
    response = await client.get("https://ddragon.leagueoflegends.com/api/versions.json")
    content = await response.json()
    latest_ddragon_ver = content[0]

    response = await client.get(
        f"https://ddragon.leagueoflegends.com/cdn/{latest_ddragon_ver}/data/en_US/champion.json",
    )
    content = await response.json()
    return content["data"]


async def get_champion_name(
    client: aiohttp.ClientSession,
    champion_id: int,
) -> str | None:
    ddragon_data = await _get_champion_list(client)
    name = ""
    for item in ddragon_data.values():
        if item["key"] == str(champion_id):
            name = item["id"]
    if name:
        return name
    return None


async def on_message(event: str | bytes, conn: Any) -> None:
    try:
        _data = json.loads(event)[2]
        data = _data["data"]
        local_p_cell_id = data["localPlayerCellId"]
        my_team = data["myTeam"]
        champion_id = 0
        summoner_id = 0
        assigned_position = None
        for p in my_team:
            if p["cellId"] == local_p_cell_id:
                logger.debug(p)
                if str(p["championId"]).strip() != "0":
                    champion_id = p["championId"]
                else:
                    champion_id = p["championPickIntent"]
                assigned_position = p["assignedPosition"]
                summoner_id = p["summonerId"]
        champion_name = (
            await get_champion_name(conn, champion_id) if champion_id else None
        )
        role = ROLES.get(assigned_position) if assigned_position is not None else None
        itemsets_payload, perks_payload, spells_payload = await get_mobalytics_info(
            champion_name, role, conn, champion_id, summoner_id
        )
        global _last_champion_id
        logger.debug(f"{champion_id} // {_last_champion_id}")
        if _last_champion_id != champion_id:
            await asyncio.gather(
                send_itemsets(conn, itemsets_payload),
                send_perks(conn, perks_payload),
                send_spells(conn, spells_payload),
            )
        _last_champion_id = champion_id
    except (json.decoder.JSONDecodeError, KeyError, TypeError, IndexError) as e:
        logger.exception(e)
        pass
    except KeyboardInterrupt:
        raise


async def send_itemsets(
    client: aiohttp.ClientSession,
    payload: Payload_ItemSets,
) -> None:
    await client.put(
        url=payload.endpoint_put,
        json=payload.asdict(),
        ssl=context,
    )


async def send_perks(client: aiohttp.ClientSession, payload: Payload_Perks) -> None:
    with contextlib.suppress(KeyError):
        response = await client.get(
            url=payload.endpoint_get,
            ssl=context,
        )
        content = await response.json()
        page_id = content["id"]
        if page_id:
            payload.endpoint_delete = page_id
            await client.delete(
                url=payload.endpoint_delete,
                ssl=context,
            )

    await client.post(
        url=payload.endpoint_post,
        json=payload.asdict(),
        ssl=context,
    )


async def send_spells(client: aiohttp.ClientSession, payload: Payload_Spells) -> None:
    await client.patch(
        url=payload.endpoint_patch,
        json=payload.asdict(),
        ssl=context,
    )


def find_proc_by_name(name: str | list[str]) -> Any:
    if type(name) is str:
        name = list(name)
    for proc in psutil.process_iter():
        if proc.name() in name:
            return proc.exe()
    return None


async def start() -> None:
    exe = find_proc_by_name(["LeagueClient.exe", "LeagueClientUx.exe"])
    if exe is None:
        msg = "league.exe not found."
        raise RuntimeError(msg)
    lockfile = _lcu_lockfile(exe)
    lockfile_data = _lcu_wss(lockfile)
    https = lockfile_data["https"]
    wss = lockfile_data["wss"]
    auth = lockfile_data["auth"]
    header = {"Authorization": f"Basic {auth}"}

    try:
        async with websockets.connect(
            uri=wss,
            additional_headers=header,
            ssl=context,
        ) as ws:
            await ws.send('[2,"0", "GetLolSummonerV1CurrentSummoner"]')
            json.loads(await ws.recv())
            await ws.send('[5, "OnJsonApiEvent_lol-champ-select_v1_session"]')
            async with aiohttp.ClientSession(base_url=https, headers=header) as conn:
                async for event in ws:
                    await on_message(event, conn)
    except (
        KeyboardInterrupt,
        asyncio.exceptions.CancelledError,
        websockets.exceptions.ConnectionClosedError,
    ) as e:
        logger.exception(e)
        pass

    return



from __future__ import annotations

from dataclasses import asdict, dataclass
from typing import Any


@dataclass
class Payload:
    def asdict(self) -> dict[str, Any]:
        return asdict(self)


@dataclass
class Payload_ItemSets(Payload):
    _endpoint_put_template = "/lol-item-sets/v1/item-sets/{accountId}/sets"
    accountId: int
    itemSets: list[ItemSet] | None = None
    timestamp: int = 1
    itemsets_css = ".m-l9l2ov > .m-yare96 > .m-1d3w5wq"

    @property
    def endpoint_put(self) -> str:
        return self._endpoint_put_template.format(accountId=self.accountId)


@dataclass
class ItemSet:
    associatedChampions: list[int]
    blocks: list[Block]
    title: str


@dataclass
class Block:
    items: list[Item]
    type: str


@dataclass
class Item:
    count: int
    id: str


@dataclass
class Payload_Perks(Payload):
    endpoint_get = "/lol-perks/v1/currentpage"
    _endpoint_delete_template = "/lol-perks/v1/pages/{pageId}"
    endpoint_post = "/lol-perks/v1/pages"

    main_perks_css = ".m-68x97p"
    selected_perks_css = (".m-1iebrlh", ".m-1nx2cdb", ".m-1u3ui07")

    name: str
    primaryStyleId: int
    subStyleId: int
    current: bool
    selectedPerkIds: list[int] | None = None

    _page_id: str | None = None

    @property
    def endpoint_delete(self) -> str:
        return self._endpoint_delete_template.format(pageId=self._page_id)

    @endpoint_delete.setter
    def endpoint_delete(self, page_id: str) -> None:
        self._page_id = page_id


@dataclass
class Payload_Spells(Payload):
    endpoint_patch = "/lol-champ-select/v1/session/my-selection"
    spell1Id: int
    spell2Id: int
    selectedSkinId: int

    spells_css = ".m-d3vnz1"


@dataclass
class EventSchema:
    actions: list[Action]
    localPlayerCellId: int
    myTeam: list[Ally]


@dataclass
class Action:
    actorCellId: int
    championId: int
    completed: bool
    type: str


@dataclass
class Ally:
    assignedPosition: str
    cellId: int
    championId: int
    selectedSkinId: int
    summonerId: int
    wardSkinId: int
